/**
 * @file
 * Builds the bidder adapters for the DFP tags.
 */

(function ($, Drupal) {

  'use strict';

  Drupal.behaviors.prebidjsDfp = {
    attach: function (context, settings) {

      var pbjs = window.pbjs || {};
      // The tags are generated by the DFP module.
      var tags = settings.dfpTags || {};
      // Gather our global settings.
      var prebidSettings = Drupal.settings.prebidSettings || {};

      pbjs.retries = 5;

      // If pbjs isn't loaded then there's no point in continuing.
      if ($.isEmptyObject(pbjs) || $.isEmptyObject(tags)) {
        throw new Error('Required library or data is missing!');
      }

      pbjs.que = window.pbjs.que || [];

      // Create an event that can be triggered when bidders are added.
      var prebidBidsEvent = $.Event('setPrebidBidders');

      /**
       * The prebid object.
       *
       * @type {{
       * prebidDefinePriceGranularity: Drupal.settings.prebid.prebidDefinePriceGranularity,
       * prebidBuildAdUnits: Drupal.settings.prebid.prebidBuildAdUnits,
       * prebidSendBidRequest: Drupal.settings.prebid.prebidSendBidRequest
       * }}
       */
      Drupal.settings.prebid = {
        /**
         * Documentation can be found here:
         * http://prebid.org/prebid-mobile/adops-price-granularity.html
         *
         * @returns {{buckets: Array}}
         */
        prebidDefinePriceGranularity: function () {
          var pricing = {
            'buckets': []
          };
          prebidSettings.buckets.forEach(function (item) {
            // The price granularity buckets is an array of objects.
            pricing.buckets.push(item);
          });
          return pricing;
        },

        /**
         * Documentation can be found here:
         * http://prebid.org/dev-docs/adunit-reference.html
         *
         * @param tags
         *   An object containing all DFP tags that will be rendered. Before
         *   they are rendered they need "Prebid ad units" defined.
         */
        prebidBuildAdUnits: function (tags) {

          for (var tag in tags) {
            // We only want to add a bidder if the tag has been saved with bidder
            // adapter data.
            if (tags.hasOwnProperty(tag)) {
              var currentTag = $.extend(true, {}, tags[tag]);
              var bidders = currentTag.bidderAdapters;
              var adUnit = {};

              adUnit.mediaTypes = {
                banner: {
                  sizes: []
                }
              };

              adUnit.code = currentTag.placeholder_id;

              // Add the ad sizes.
              if ($.isArray(currentTag.size[0])) {
                adUnit.mediaTypes.banner.sizes = adUnit.sizes = currentTag.size;
              }
              else {
                var sizes = [];
                sizes.push(currentTag.size);
                adUnit.mediaTypes.banner.sizes = adUnit.sizes = sizes;
              }

              adUnit.bids = [];

              for (var key in bidders) {
                var bid = {};
                if (bidders.hasOwnProperty(key)) {

                  var bidder = $.extend(true, {}, bidders[key]);

                  if (bidder.tags.length && $.inArray(currentTag.machinename, bidder.tags) > -1) {

                    bid = {
                      bidder: bidder.name,
                      params: bidder.params,
                      push: true
                    };

                    // Trigger the setPrebidConfig so others can modify the configuration.
                    $(document).trigger(prebidBidsEvent, [{'bid':bid, 'adUnit':adUnit, 'tag':currentTag}]);

                    if (bid.push) {
                      adUnit.bids.push(bid);
                    }
                  }
                }
              }

              if (adUnit.bids.length) {
                prebidSettings.adUnits.push(adUnit);
              }
            }
          }
        },

        /**
         * This method should be used when the ads are being refreshed after
         * the initial page load.
         *
         * This will ensure the existing adUnits are cleared, new adUnits
         * are built and added and then the bids are requested.
         *
         * @param slotsToRefresh
         *   Indexed array of dfp defined slots to refresh.
         * @param adUnitCodes
         *   Indexed array where the values are the div ids the tags will be
         *   rendered in. This is the machine name of the tag with 'dfp-ad-'
         *   prepended to it.
         * @param dfpTags
         *   Associative array of raw dfp tags keyed by the tag's machine name.
         */
        prebidRefreshBids: function (slotsToRefresh, adUnitCodes, dfpTags) {
          // Clear any existing prebid adUnits.
          Drupal.settings.prebidSettings.adUnits = [];

          // Rebuild the adUnits for pbjs.
          Drupal.settings.prebid.prebidBuildAdUnits(dfpTags);

          // Add the ad units to pbjs.
          pbjs.que.push(function() {
            pbjs.addAdUnits(Drupal.settings.prebidSettings.adUnits);
          });

          // Send a bid request for the newly built ads.
          Drupal.settings.prebid.prebidSendBidRequest(slotsToRefresh, adUnitCodes);
        },

        /**
         * Set the targeting and refresh the slots.
         *
         * @param slotsToRefresh
         *   An array of slots (tags) that will be refreshed|rendered.
         * @param adUnitCodes
         *   An array of
         * @param single
         *   Truthy value that determines if this method will be called more
         *   than once per page load.
         *
         */
        prebidSendBidRequest: function (slotsToRefresh, adUnitCodes, single) {
          if (single && pbjs.initAdserverSet) return;

          // We don't want to call DFP before it is ready.
          if (!googletag.pubadsReady && pbjs.retries <= 20) {
            setTimeout(Drupal.settings.prebid.prebidSendBidRequest(slotsToRefresh, adUnitCodes), 100); //poll ms can be adjusted as desired.
            pbjs.retries++;
            return;
          }

          pbjs.que.push(function() {
            pbjs.requestBids({
              // timeout: Drupal.settings.prebidSettings.global.timeout,
              timeout: Drupal.settings.prebidSettings.global.timeout,
              adUnitCodes: adUnitCodes,
              bidsBackHandler: function(bidResponses) {
                console.log("Bid responses:", bidResponses);
                pbjs.setTargetingForGPTAsync(adUnitCodes);
                googletag.pubads().refresh(slotsToRefresh,  {changeCorrelator: false});
              }
            });
          });

          if (single) pbjs.initAdserverSet = true;

          // Log the slots that were refreshed.
          Drupal.settings.dfp.logSlotsRefreshed(adUnitCodes);
        }

      };
    }
  };

})(jQuery, Drupal);
